@@ -156,18 +156,19 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 
 	done := map[string]bool{}
 	for _, selector := range selectors {
-		if _, ok := done[selector.String()]; ok {
+		s := selector.String()
+		if _, ok := done[s]; ok {
 			continue
 		}
 
-		done[selector.String()] = true
+		done[s] = true
 
 		if isDisabled(entry.Rule, selector) {
-			done[selector.String()] = true
+			done[s] = true
 			continue
 		}
 		if isSnoozed(entry.Rule, selector) {
-			done[selector.String()] = true
+			done[s] = true
 			continue
 		}
 
@@ -200,9 +201,9 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 					}
 				}
 				if arEntry != nil {
-					slog.Debug(
+					slog.LogAttrs(ctx, slog.LevelDebug,
 						"Metric is provided by alerting rule",
-						slog.String("selector", selector.String()),
+						slog.String("selector", s),
 						slog.String("path", arEntry.Path.Name),
 					)
 				} else {
@@ -215,7 +216,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						Severity: Bug,
 						Diagnostics: []diags.Diagnostic{
 							{
-								Message:     fmt.Sprintf("`%s` metric is generated by alerts but didn't found any rule named `%s`.", selector.String(), alertname),
+								Message:     fmt.Sprintf("`%s` metric is generated by alerts but didn't found any rule named `%s`.", s, alertname),
 								Pos:         expr.Value.Pos,
 								FirstColumn: int(selector.PosRange.Start) + 1,
 								LastColumn:  int(selector.PosRange.End),
@@ -244,30 +245,30 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 		}
 
 		// 1. If foo{bar, baz} is there -> GOOD
-		slog.Debug("Checking if selector returns anything", slog.String("check", c.Reporter()), slog.String("selector", selector.String()))
-		count, err := c.instantSeriesCount(ctx, fmt.Sprintf("count(%s)", selector.String()))
+		slog.LogAttrs(ctx, slog.LevelDebug, "Checking if selector returns anything", slog.String("check", c.Reporter()), slog.String("selector", s))
+		count, err := c.instantSeriesCount(ctx, wrapExpr(s, "count"))
 		if err != nil {
 			problems = append(problems, problemFromError(err, entry.Rule, c.Reporter(), c.prom.Name(), Bug))
 			continue
 		}
 		if count > 0 {
-			slog.Debug("Found series, skipping further checks", slog.String("check", c.Reporter()), slog.String("selector", selector.String()))
+			slog.LogAttrs(ctx, slog.LevelDebug, "Found series, skipping further checks", slog.String("check", c.Reporter()), slog.String("selector", s))
 			continue
 		}
 
-		promUptime, err := c.prom.RangeQuery(ctx, fmt.Sprintf("count(%s)", c.prom.UptimeMetric()), params)
+		promUptime, err := c.prom.RangeQuery(ctx, wrapExpr(c.prom.UptimeMetric(), "count"), params)
 		if err != nil {
-			slog.Warn("Cannot detect Prometheus uptime gaps", slog.Any("err", err), slog.String("name", c.prom.Name()))
+			slog.LogAttrs(ctx, slog.LevelWarn, "Cannot detect Prometheus uptime gaps", slog.Any("err", err), slog.String("name", c.prom.Name()))
 		}
-		if promUptime != nil && promUptime.Series.Ranges.Len() == 0 {
-			slog.Warn(
+		if promUptime != nil && len(promUptime.Series.Ranges) == 0 {
+			slog.LogAttrs(ctx, slog.LevelWarn,
 				"No results for Prometheus uptime metric, you might have set uptime config option to a missing metric, please check your config",
 				slog.String("name", c.prom.Name()),
 				slog.String("metric", c.prom.UptimeMetric()),
 			)
 		}
-		if promUptime == nil || promUptime.Series.Ranges.Len() == 0 {
-			slog.Warn(
+		if promUptime == nil || len(promUptime.Series.Ranges) == 0 {
+			slog.LogAttrs(ctx, slog.LevelWarn,
 				"Using dummy Prometheus uptime metric results with no gaps",
 				slog.String("name", c.prom.Name()),
 				slog.String("metric", c.prom.UptimeMetric()),
@@ -292,14 +293,15 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 		}
 
 		bareSelector := stripLabels(selector)
+		bareSelectorString := bareSelector.String()
 
-		if bareSelector.String() == "" {
+		if bareSelectorString == "" {
 			continue
 		}
 
 		// 2. If foo was NEVER there -> BUG
-		slog.Debug("Checking if base metric has historical series", slog.String("check", c.Reporter()), slog.String("selector", (&bareSelector).String()))
-		trs, err := c.prom.RangeQuery(ctx, fmt.Sprintf("count(%s)", bareSelector.String()), params)
+		slog.LogAttrs(ctx, slog.LevelDebug, "Checking if base metric has historical series", slog.String("check", c.Reporter()), slog.String("selector", bareSelectorString))
+		trs, err := c.prom.RangeQuery(ctx, wrapExpr(bareSelectorString, "count"), params)
 		if err != nil {
 			problems = append(problems, problemFromError(err, entry.Rule, c.Reporter(), c.prom.Name(), Bug))
 			continue
@@ -308,17 +310,17 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 		if len(trs.Series.Ranges) == 0 {
 			// Check if we have recording rule that provides this metric before we give up
 			var rrEntry *discovery.Entry
-			for _, entry := range entries {
-				if entry.Rule.RecordingRule != nil &&
-					entry.Rule.Error.Err == nil &&
-					entry.Rule.RecordingRule.Record.Value == bareSelector.String() {
-					rrEntry = &entry
+			for ei := range entries {
+				if entries[ei].Rule.RecordingRule != nil &&
+					entries[ei].Rule.Error.Err == nil &&
+					entries[ei].Rule.RecordingRule.Record.Value == bareSelectorString {
+					rrEntry = &entries[ei]
 					break
 				}
 			}
 			if rrEntry != nil {
 				// Validate recording rule instead
-				slog.Debug("Metric is provided by recording rule", slog.String("selector", (&bareSelector).String()), slog.String("path", rrEntry.Path.Name))
+				slog.LogAttrs(ctx, slog.LevelDebug, "Metric is provided by recording rule", slog.String("selector", bareSelectorString), slog.String("path", rrEntry.Path.Name))
 				problems = append(problems, Problem{
 					Anchor:   AnchorAfter,
 					Lines:    expr.Value.Pos.Lines(),
@@ -329,7 +331,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 					Diagnostics: []diags.Diagnostic{
 						{
 							Message: fmt.Sprintf("%s didn't have any series for `%s` metric in the last %s but found recording rule that generates it, skipping further checks.",
-								promText(c.prom.Name(), trs.URI), bareSelector.String(), sinceDesc(trs.Series.From)),
+								promText(c.prom.Name(), trs.URI), bareSelectorString, sinceDesc(trs.Series.From)),
 							Pos:         expr.Value.Pos,
 							FirstColumn: int(selector.PosRange.Start) + 1,
 							LastColumn:  int(selector.PosRange.End),
@@ -342,15 +344,15 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 
 			text, severity := c.textAndSeverity(
 				settings,
-				bareSelector.String(),
+				bareSelectorString,
 				fmt.Sprintf("%s didn't have any series for `%s` metric in the last %s.",
 					promText(c.prom.Name(), trs.URI),
-					bareSelector.String(),
+					bareSelectorString,
 					sinceDesc(trs.Series.From),
 				),
 				Bug,
 			)
-			if details, shouldReport := c.checkOtherServer(ctx, selector.String(), settings); shouldReport {
+			if details, shouldReport := c.checkOtherServer(ctx, s, settings); shouldReport {
 				problems = append(problems, Problem{
 					Anchor:   AnchorAfter,
 					Lines:    expr.Value.Pos.Lines(),
@@ -369,7 +371,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 					},
 				})
 			}
-			slog.Debug("No historical series for base metric", slog.String("check", c.Reporter()), slog.String("selector", (&bareSelector).String()))
+			slog.LogAttrs(ctx, slog.LevelDebug, "No historical series for base metric", slog.String("check", c.Reporter()), slog.String("selector", bareSelectorString))
 			continue
 		}
 
@@ -377,8 +379,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 		for _, name := range labelNames {
 			l := stripLabels(selector)
 			l.LabelMatchers = append(l.LabelMatchers, labels.MustNewMatcher(labels.MatchRegexp, name, ".+"))
-			slog.Debug("Checking if base metric has historical series with required label", slog.String("check", c.Reporter()), slog.String("selector", (&l).String()), slog.String("label", name))
-			trsLabelCount, err := c.prom.RangeQuery(ctx, fmt.Sprintf("absent(%s)", l.String()), params)
+			ls := l.String()
+
+			slog.LogAttrs(ctx, slog.LevelDebug, "Checking if base metric has historical series with required label", slog.String("check", c.Reporter()), slog.String("selector", ls), slog.String("label", name))
+			trsLabelCount, err := c.prom.RangeQuery(ctx, wrapExpr(ls, "absent"), params)
 			if err != nil {
 				problems = append(problems, problemFromError(err, entry.Rule, c.Reporter(), c.prom.Name(), Bug))
 				continue
@@ -397,7 +401,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 				continue
 			}
 
-			if trsLabelCount.Series.Ranges.Len() == 1 && len(trsLabelCount.Series.Gaps) == 0 {
+			if len(trsLabelCount.Series.Ranges) == 1 && len(trsLabelCount.Series.Gaps) == 0 {
 				problems = append(problems, Problem{
 					Anchor:   AnchorAfter,
 					Lines:    expr.Value.Pos.Lines(),
@@ -409,7 +413,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						{
 							Message: fmt.Sprintf(
 								"%s has `%s` metric but there are no series with `%s` label in the last %s.",
-								promText(c.prom.Name(), trsLabelCount.URI), bareSelector.String(), name, sinceDesc(trsLabelCount.Series.From)),
+								promText(c.prom.Name(), trsLabelCount.URI), bareSelectorString, name, sinceDesc(trsLabelCount.Series.From)),
 							Pos:         expr.Value.Pos,
 							FirstColumn: int(selector.PosRange.Start) + 1,
 							LastColumn:  int(selector.PosRange.End),
@@ -417,7 +421,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						},
 					},
 				})
-				slog.Debug("No historical series with label used for the query", slog.String("check", c.Reporter()), slog.String("selector", (&l).String()), slog.String("label", name))
+				slog.LogAttrs(ctx, slog.LevelDebug, "No historical series with label used for the query", slog.String("check", c.Reporter()), slog.String("selector", ls), slog.String("label", name))
 			}
 		}
 		if len(problems) > 0 {
@@ -435,10 +439,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 			}
 
 			if !newest(trs.Series.Ranges).Before(trs.Series.Until.Add(minAge * -1)) {
-				slog.Debug(
+				slog.LogAttrs(ctx, slog.LevelDebug,
 					"Series disappeared from prometheus but for less then configured min-age",
 					slog.String("check", c.Reporter()),
-					slog.String("selector", selector.String()),
+					slog.String("selector", s),
 					slog.String("min-age", output.HumanizeDuration(minAge)),
 					slog.String("last-seen", sinceDesc(newest(trs.Series.Ranges))),
 				)
@@ -447,10 +451,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 
 			text, severity := c.textAndSeverity(
 				settings,
-				bareSelector.String(),
+				bareSelectorString,
 				fmt.Sprintf(
 					"%s doesn't currently have `%s`, it was last present %s ago.",
-					promText(c.prom.Name(), trs.URI), bareSelector.String(), sinceDesc(newest(trs.Series.Ranges))),
+					promText(c.prom.Name(), trs.URI), bareSelectorString, sinceDesc(newest(trs.Series.Ranges))),
 				Bug,
 			)
 			problems = append(problems, Problem{
@@ -470,7 +474,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 					},
 				},
 			})
-			slog.Debug("Series disappeared from prometheus", slog.String("check", c.Reporter()), slog.String("selector", (&bareSelector).String()))
+			slog.LogAttrs(ctx, slog.LevelDebug, "Series disappeared from prometheus", slog.String("check", c.Reporter()), slog.String("selector", bareSelectorString))
 			continue
 		}
 
@@ -484,16 +488,18 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 			if c.isLabelValueIgnored(settings, entry.Rule, selector, lm.Name) {
 				continue
 			}
+			lms := lm.String()
 
 			pos := findMatcherPos(expr.Value.Value, selector.PosRange, lm)
-			labelSelector := promParser.VectorSelector{
+			labelSelector := &promParser.VectorSelector{
 				Name:          metricName,
 				LabelMatchers: []*labels.Matcher{lm},
 			}
-			addNameSelectorIfNeeded(&labelSelector, selector.LabelMatchers)
-			slog.Debug("Checking if there are historical series matching filter", slog.String("check", c.Reporter()), slog.String("selector", (&labelSelector).String()), slog.String("matcher", lm.String()))
+			addNameSelectorIfNeeded(labelSelector, selector.LabelMatchers)
+			labelSelectorString := labelSelector.String()
+			slog.LogAttrs(ctx, slog.LevelDebug, "Checking if there are historical series matching filter", slog.String("check", c.Reporter()), slog.String("selector", labelSelectorString), slog.String("matcher", lms))
 
-			trsLabel, err := c.prom.RangeQuery(ctx, fmt.Sprintf("count(%s)", labelSelector.String()), params)
+			trsLabel, err := c.prom.RangeQuery(ctx, wrapExpr(labelSelectorString, "count"), params)
 			if err != nil {
 				problems = append(problems, problemFromError(err, entry.Rule, c.Reporter(), c.prom.Name(), Bug))
 				continue
@@ -504,10 +510,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 			if len(trsLabel.Series.Ranges) == 0 {
 				text, severity := c.textAndSeverity(
 					settings,
-					bareSelector.String(),
+					bareSelectorString,
 					fmt.Sprintf(
 						"%s has `%s` metric with `%s` label but there are no series matching `{%s}` in the last %s.",
-						promText(c.prom.Name(), trsLabel.URI), bareSelector.String(), lm.Name, lm.String(), sinceDesc(trs.Series.From)),
+						promText(c.prom.Name(), trsLabel.URI), bareSelectorString, lm.Name, lms, sinceDesc(trs.Series.From)),
 					Bug,
 				)
 				problems = append(problems, Problem{
@@ -527,8 +533,8 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						},
 					},
 				})
-				slog.Debug("No historical series matching filter used in the query",
-					slog.String("check", c.Reporter()), slog.String("selector", selector.String()), slog.String("matcher", lm.String()))
+				slog.LogAttrs(ctx, slog.LevelDebug, "No historical series matching filter used in the query",
+					slog.String("check", c.Reporter()), slog.String("selector", s), slog.String("matcher", lms))
 				continue
 			}
 
@@ -563,10 +569,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 				}
 
 				if !newest(trsLabel.Series.Ranges).Before(trsLabel.Series.Until.Add(minAge * -1)) {
-					slog.Debug(
+					slog.LogAttrs(ctx, slog.LevelDebug,
 						"Series disappeared from prometheus but for less then configured min-age",
 						slog.String("check", c.Reporter()),
-						slog.String("selector", selector.String()),
+						slog.String("selector", s),
 						slog.String("min-age", output.HumanizeDuration(minAge)),
 						slog.String("last-seen", sinceDesc(newest(trsLabel.Series.Ranges))),
 					)
@@ -575,10 +581,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 
 				text, severity := c.textAndSeverity(
 					settings,
-					bareSelector.String(),
+					bareSelectorString,
 					fmt.Sprintf(
 						"%s has `%s` metric but doesn't currently have series matching `{%s}`, such series was last present %s ago.",
-						promText(c.prom.Name(), trs.URI), bareSelector.String(), lm.String(), sinceDesc(newest(trsLabel.Series.Ranges))),
+						promText(c.prom.Name(), trs.URI), bareSelectorString, lms, sinceDesc(newest(trsLabel.Series.Ranges))),
 					Bug,
 				)
 				problems = append(problems, Problem{
@@ -598,11 +604,11 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						},
 					},
 				})
-				slog.Debug(
+				slog.LogAttrs(ctx, slog.LevelDebug,
 					"Series matching filter disappeared from prometheus",
 					slog.String("check", c.Reporter()),
-					slog.String("selector", selector.String()),
-					slog.String("matcher", lm.String()),
+					slog.String("selector", s),
+					slog.String("matcher", lms),
 				)
 				continue
 			}
@@ -620,7 +626,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						{
 							Message: fmt.Sprintf(
 								"Metric `%s` with label `{%s}` is only sometimes present on %s with average life span of %s.",
-								bareSelector.String(), lm.String(), promText(c.prom.Name(), trs.URI),
+								bareSelectorString, lms, promText(c.prom.Name(), trs.URI),
 								output.HumanizeDuration(avgLife(trsLabel.Series.Ranges))),
 							Pos:         expr.Value.Pos,
 							FirstColumn: int(pos.Start) + 1,
@@ -629,11 +635,11 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 						},
 					},
 				})
-				slog.Debug(
+				slog.LogAttrs(ctx, slog.LevelDebug,
 					"Series matching filter are only sometimes present",
 					slog.String("check", c.Reporter()),
-					slog.String("selector", selector.String()),
-					slog.String("matcher", lm.String()),
+					slog.String("selector", s),
+					slog.String("matcher", lms),
 				)
 			}
 		}
@@ -654,7 +660,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 					{
 						Message: fmt.Sprintf(
 							"Metric `%s` is only sometimes present on %s with average life span of %s in the last %s.",
-							bareSelector.String(), promText(c.prom.Name(), trs.URI), output.HumanizeDuration(avgLife(trs.Series.Ranges)), sinceDesc(trs.Series.From)),
+							bareSelectorString, promText(c.prom.Name(), trs.URI), output.HumanizeDuration(avgLife(trs.Series.Ranges)), sinceDesc(trs.Series.From)),
 						Pos:         expr.Value.Pos,
 						FirstColumn: int(selector.PosRange.Start) + 1,
 						LastColumn:  int(selector.PosRange.End),
@@ -662,10 +668,10 @@ func (c SeriesCheck) Check(ctx context.Context, entry discovery.Entry, entries [
 					},
 				},
 			})
-			slog.Debug(
+			slog.LogAttrs(ctx, slog.LevelDebug,
 				"Metric only sometimes present",
 				slog.String("check", c.Reporter()),
-				slog.String("selector", (&bareSelector).String()),
+				slog.String("selector", bareSelectorString),
 			)
 		}
 	}
@@ -737,7 +743,7 @@ func (c SeriesCheck) checkOtherServer(ctx context.Context, query string, setting
 	var tested, matches, skipped int
 	for _, prom := range servers {
 		if time.Since(start) >= settings.fallbackTimeout {
-			slog.Debug("Time limit reached for checking if metric exists on any other Prometheus server",
+			slog.LogAttrs(ctx, slog.LevelDebug, "Time limit reached for checking if metric exists on any other Prometheus server",
 				slog.String("check", c.Reporter()),
 				slog.String("selector", query),
 			)
@@ -746,14 +752,14 @@ func (c SeriesCheck) checkOtherServer(ctx context.Context, query string, setting
 			break
 		}
 
-		slog.Debug("Checking if metric exists on any other Prometheus server",
+		slog.LogAttrs(ctx, slog.LevelDebug, "Checking if metric exists on any other Prometheus server",
 			slog.String("check", c.Reporter()),
 			slog.String("name", prom.Name()),
 			slog.String("selector", query),
 		)
 
 		tested++
-		qr, err := prom.Query(ctx, fmt.Sprintf("count(%s)", query))
+		qr, err := prom.Query(ctx, wrapExpr(query, "count"))
 		if err != nil {
 			continue
 		}
@@ -886,7 +892,7 @@ func (c SeriesCheck) isLabelValueIgnored(settings *PromqlSeriesSettings, rule pa
 			continue
 		}
 		if slices.Contains(names, labelName) {
-			slog.Debug("Label check disabled globally via config", slog.String("label", labelName))
+			slog.LogAttrs(context.Background(), slog.LevelDebug, "Label check disabled globally via config", slog.String("label", labelName))
 			return true
 		}
 	}
@@ -902,7 +908,7 @@ func (c SeriesCheck) isLabelValueIgnored(settings *PromqlSeriesSettings, rule pa
 			}
 		}
 		if labelName == value {
-			slog.Debug("Label check disabled by comment", slog.String("selector", selector.String()), slog.String("label", labelName))
+			slog.LogAttrs(context.Background(), slog.LevelDebug, "Label check disabled by comment", slog.String("selector", selector.String()), slog.String("label", labelName))
 			return true
 		}
 	}
@@ -912,7 +918,7 @@ func (c SeriesCheck) isLabelValueIgnored(settings *PromqlSeriesSettings, rule pa
 func (c SeriesCheck) textAndSeverity(settings *PromqlSeriesSettings, name, text string, s Severity) (string, Severity) {
 	for _, re := range settings.ignoreMetricsRe {
 		if name != "" && re.MatchString(name) {
-			slog.Debug(
+			slog.LogAttrs(context.Background(), slog.LevelDebug,
 				"Metric matches check ignore rules",
 				slog.String("check", c.Reporter()),
 				slog.String("metric", name),
@@ -1038,7 +1044,7 @@ func isSnoozed(rule parser.Rule, selector *promParser.VectorSelector) bool {
 
 func matchSelectorToMetric(selector *promParser.VectorSelector, metric string) (bool, bool) {
 	// Try full string or name match first.
-	if metric == selector.String() || metric == selector.Name {
+	if metric == selector.Name || metric == selector.String() {
 		return true, true
 	}
 	// Then try matchers.
